@@ -9,7 +9,7 @@ import gitlab
 from enum import Enum
 import csv
 
-from result import Ok, Err
+from result import Ok, Err, Result
 
 KEY_STUDENT_ID = 'id'
 KEY_STUDENT_NAME_FIRST = 'first_name'
@@ -184,7 +184,7 @@ class Ep2Group:
 
 class Ep2:
 
-    def __init__(self, verbose=False, config=None):
+    def __init__(self, verbose: bool = False, config: configparser.ConfigParser = None, sudo: str = None):
         if config is None:
             self.config = create_config()
         else:
@@ -192,6 +192,10 @@ class Ep2:
 
         self.gitlab = create_gitlab(config=self.config)
         self.verbose = verbose
+        self.sudo = sudo
+
+        if self.sudo is not None:
+            self.print_sudo_header()
 
     def tag_project(self, project, tag):
         """Tags the last commit of a given project with 'tag'."""
@@ -212,7 +216,7 @@ class Ep2:
 
         # TODO check timing constraints? e.g. commit time can't be later than 5 minutes after hand-in?
         try:
-            project.tags.create({'tag_name': tag, 'ref': commit_id})  # create tag for commit
+            project.tags.create({'tag_name': tag, 'ref': commit_id}, sudo=self.sudo)  # create tag for commit
         except gitlab.exceptions.GitlabCreateError:  # tag already exists, fail gracefully
             click.secho('Tag already exists in project', fg='red')
             return False
@@ -242,7 +246,8 @@ class Ep2:
             except gitlab.exceptions.GitlabGetError:
                 if self.verbose:
                     click.secho('No branch ' + tag + ' found! Aborting clone/pull', fg='red')
-                return Err('no branch/tag ' + tag + ' found in repository %s' % project.name_with_namespace.replace(' ', ''))
+                return Err(
+                    'no branch/tag ' + tag + ' found in repository %s' % project.name_with_namespace.replace(' ', ''))
 
         if os.path.exists(repo_dir):  # dir exists, no need to clone
             repo = git.Repo(repo_dir)
@@ -347,6 +352,41 @@ class Ep2:
 
         raise Exception('no test info available for exercise %d' % ue)
 
+    def uebung_dir(self):
+        return os.path.join(self.config.get('Local', 'GitHome'), "uebung")
+
+    def create_project_issue(self, project:str, title:str, descrition:str):
+        proj = self.gitlab.projects.get(project, lazy=True)  # fetch project (gitlab) and add issue
+
+        proj.issues.create({'title': title, 'description': descrition}, sudo=self.sudo)
+
+    def print_sudo_header(self):
+        user = self.gitlab.users.list(username=self.sudo)[0]
+        click.secho('*' * 60, fg='red', bold=True)
+        click.secho('*' + ' ' * 27 + 'SUDO' + ' ' * 27 + '*', fg='red', bold=True)
+        click.secho('*' + ' ' * 58 + '*', fg='red', bold=True)
+        click.secho('* {:^56s} *'.format('Performing actions as {:s}'.format(user.name)), fg='red', bold=True)
+        click.secho('*' * 60, fg='red', bold=True)
+
+
+def verify_and_normalize_student_id(student_id) -> Result:
+    import re
+    if re.match('\\d*', student_id) and len(student_id) <= 8:
+        return Ok('0' * (8 - len(student_id)) + student_id)
+    else:
+        return Err('invalid student id')
+
+
+def validate_student_id(ctx, param, value):
+    if value is None:
+        return None
+
+    res = verify_and_normalize_student_id(value)
+    if res.is_ok():
+        return res.ok()
+    else:
+        raise click.BadParameter(res.err())
+
 
 def create_gitlab(config):
     """Opens a gitlab instance with the supplied config.
