@@ -80,14 +80,16 @@ def pre_eval_csv_fieldnames_ta():
 
 @click.group()
 @click.option("--verbose/--silent", default=False, help='output extra information about the current steps')
+@click.option("--sudo", required=False, default=None, help='Perform actions as a different user, ONLY usable by admins')
 @click.pass_context
-def cli(ctx, verbose):
+def cli(ctx, verbose, sudo):
     """Utility for ep2 tutors to perform pre evaluations of submitted exercises"""
     if verbose:
         click.echo("[DEBUG] Verbose output enabled!")
 
     ctx.ensure_object(dict)
     ctx.obj['VERBOSE'] = verbose
+    ctx.obj['SUDO'] = sudo
 
 
 @cli.command()
@@ -98,15 +100,14 @@ def checkout(ctx, group, ue: int):
     """Checks out all repositories of a group for a given exercise."""
     curr_ue_type = ue_type(ue)
 
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
 
     students_file = group.students_csv()  # list of all students for this group
     pre_eval_file = group.pre_eval_csv(ue)
     f_info = FileInformation('Checkout')
 
-    uebung_path = os.path.join(ep2.config.get('Local', 'GitHome'), "uebung")
-
+    uebung_path = ep2.uebung_dir()
     errors = []
 
     with open(students_file, 'r') as infile:  # read students and create entries in attendance file
@@ -215,7 +216,8 @@ def validate_ex_type_ta(ctx, param, value):
 @click.option("--group", required=True, prompt=True, help='name of the group')
 @click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero', is_eager=True,
               type=click.INT)
-@click.option("--student", required=True, prompt=True, help='student id of the student, that should be graded')
+@click.option("--student", required=True, prompt=True, help='student id of the student, that should be graded',
+              callback=validate_student_id)
 @click.option("--student-feedback", prompt=True, default='', help='feedback for the student, will be part of the '
                                                                   'created issue')
 @click.option("--solution-remarks", prompt=True, default='', help='remarks for the lecturer (keep short)')
@@ -225,13 +227,15 @@ def validate_ex_type_ta(ctx, param, value):
 @click.option('--points', type=click.IntRange(0, 33), help='points for team exercise', callback=validate_ex_type_ta)
 @click.pass_context
 def grade(ctx, group, ue: int, student, grading, student_feedback, solution_remarks, tasks, points):
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
     f_info = FileInformation('Grade')
 
     pre_eval_csv_file = group.pre_eval_csv(ue)  # file for reading
     tmp_file = pre_eval_csv_file + '.tmp'  # file for writing
 
+    print(student)
+
     curr_ue_type = ue_type(ue)
     if curr_ue_type == ExerciseType.Team:
         if points is None:
@@ -350,7 +354,7 @@ def submit(ctx, group: str, ue: int):
 
     To prevent accidental issue creation before validation, a challenge has to be entered, that is printed
     at the top of the output."""
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
     f_info = FileInformation('Submit')
 
@@ -457,13 +461,11 @@ def submit(ctx, group: str, ue: int):
         for student_id, issue in bar:
             student_project = ep2.ue_repo(student_id)  # get project
             try:
-                proj = ep2.gitlab.projects.get(student_project, lazy=True)  # fetch project (gitlab) and add issue
                 if curr_ue_type == ExerciseType.Normal:
                     title = 'Abgabe %d' % ue
                 else:
                     title = 'Abgabe Teamaufgabe'
-
-                proj.issues.create({'title': title, 'description': issue})
+                ep2.create_project_issue(project=student_project, title=title, descrition=issue)
             except gitlab.GitlabCreateError as e:
                 exceptions += [(e, student_project)]
 
@@ -491,7 +493,7 @@ def list_grp():
 @click.pass_context
 def list_ungraded(ctx, ue, group):
     """This command lists all ungraded submission for a specific group and exercise."""
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
 
     csv_file = group.pre_eval_csv(ue)
