@@ -128,17 +128,40 @@ class StudentIndex:
         self.remaining.remove(student_id)
         self.done.append(student_id)
 
+    def auto_fill_team(self, repo_owner):
+        members = self.team_members(repo_owner)
+        auto_fill_additionals: [str] = []
+
+        for member in members:
+            while member[0] == '0':
+                member = member[1:]
+                auto_fill_additionals += [member]
+
+        return members + auto_fill_additionals
+
+    def auto_fill_index(self):
+        auto_fill_additionals: [str] = []
+
+        for member in self.remaining:
+            while member[0] == '0':
+                member = member[1:]
+                auto_fill_additionals += [member]
+
+        return self.remaining + auto_fill_additionals
+
 
 @click.group()
 @click.option("--verbose/--silent", default=False, help='output extra information about the current steps')
+@click.option("--sudo", required=False, default=None, help='Perform actions as a different user, ONLY usable by admins')
 @click.pass_context
-def cli(ctx, verbose):
+def cli(ctx, verbose, sudo):
     """Utility for ep2 tutors to keep track of submissions and grade ad-hoc exercises."""
     if verbose:
         click.echo("[DEBUG] Verbose output enabled!")
 
     ctx.ensure_object(dict)
     ctx.obj['VERBOSE'] = verbose
+    ctx.obj['SUDO'] = sudo
 
 
 def team_name(members):
@@ -152,7 +175,7 @@ def team_name(members):
 @click.confirmation_option(prompt='This will tag all repositories with the adhoc tags. Continue?')
 @click.pass_context
 def tag(ctx, group, ue: int, late):
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
 
     group.tag_group((TAG_NAME_EX_TEST % ue) if not late else (TAG_NAME_EX_TEST_LATE % ue))
@@ -169,7 +192,7 @@ def tag(ctx, group, ue: int, late):
 @click.pass_context
 def submission(ctx, group, ue, late, idea, grade, single):
     """Adds submissions of teams for a specific exercise to the corresponding CSV files."""
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
     f_info = FileInformation('Submission')
 
@@ -211,25 +234,33 @@ def submission(ctx, group, ue, late, idea, grade, single):
         while len(index.remaining) > 0:
 
             # prompt owner of working repository
-            repo_owner = click.prompt("Mat.No. Repo", prompt_suffix=':   ', default="", show_default=False,
-                                      type=click.Choice(index.remaining + [""]), show_choices=False)
-            if repo_owner == '':
+            repo_owner = prompt_manual("Mat.No. Repo", suffix=':   ', default="", show_default=False,
+                                       type=click.Choice(index.auto_fill_index() + [""]), exit_condition=[''])
+
+            if repo_owner.is_err():
                 break
 
+            repo_owner = repo_owner.ok()
+
             index.mark_done(repo_owner)
             if not single:
-                auto_fill = index.team_members(repo_owner)
+                auto_fill = index.auto_fill_team(repo_owner)
 
                 if len(index.remaining) == 0:
                     click.secho('Invalid state: no student ids remaining!')
                     exit(1)
 
                 if len(auto_fill) == 1:
-                    editor = click.prompt("Mat.No. Writer", type=click.Choice(index.remaining),
-                                          show_choices=False, default=auto_fill[0])  # prompt writer
+                    editor = prompt_manual("Mat.No. Writer", type=click.Choice(index.auto_fill_index()), default=auto_fill[0],
+                                           show_default=True, suffix=': ')
                 else:
-                    editor = click.prompt("Mat.No. Writer", type=click.Choice(index.remaining),
-                                          show_choices=False)  # prompt writer
+                    editor = prompt_manual("Mat.No. Writer", type=click.Choice(index.auto_fill_index()), show_default=True,
+                                           suffix=': ')
+
+                if editor.is_err():
+                    break
+
+                editor = editor.ok()
 
                 index.mark_done(editor)
                 if editor in auto_fill:
@@ -239,12 +270,16 @@ def submission(ctx, group, ue, late, idea, grade, single):
                 # allow for teams of 3 people
 
                 if len(auto_fill) == 1:
-                    third = click.prompt("Mat.No. Third", prompt_suffix=':  ',
-                                         type=click.Choice(index.remaining + ['none']),
-                                         show_choices=False, default=auto_fill[0])
+                    third = prompt_manual("Mat.No. Third", suffix=':  ', type=click.Choice(index.auto_fill_index() + ['none']),
+                                          default=auto_fill[0], show_default=True, exit_condition=['', 'none'])
                 else:
-                    third = click.prompt("Mat.No. Third", prompt_suffix=':  ', default='none', show_default=False,
-                                         type=click.Choice(index.remaining + ['none']), show_choices=False)
+                    third = prompt_manual("Mat.No. Third", suffix=':  ', type=click.Choice(index.auto_fill_index() + ['none']),
+                                          default='none', show_default=False, exit_condition=['', 'none'])
+
+                if third.is_err():
+                    third = None
+                else:
+                    third = third.ok()
 
                 if third is not None and third != 'none':
                     members = [(repo_owner, 'o'), (editor, 'e'), (third, 't')]
@@ -296,6 +331,28 @@ def submission(ctx, group, ue, late, idea, grade, single):
     click.echo('Submit your results using ' + click.style('adhoc submit', bold=True) + '.')
 
 
+def prompt_manual(prompt:str, suffix:str, type:click.Choice, show_default: bool, default:str = None,
+                  exit_condition: [str] = []) -> Result[str, str]:
+    prompt_result = click.prompt(prompt, prompt_suffix=suffix, show_default=show_default,
+                         type=type, show_choices=False, default=default)
+
+    if prompt_result in exit_condition:
+        return Err('exit condition reached')
+
+    verification_result = verify_and_normalize_student_id(prompt_result)
+
+    while verification_result.is_err():
+        prompt_result = click.prompt(prompt, prompt_suffix=suffix, show_default=show_default,
+                                     type=type, show_choices=False, default=default)
+
+        if prompt_result in exit_condition:
+            return Err('exit condition reached')
+
+        verification_result = verify_and_normalize_student_id(prompt_result)
+
+    return verification_result
+
+
 @cli.command()
 @click.option("--group", required=True, prompt=True, help='name of the group')
 @click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero', type=click.INT)
@@ -307,7 +364,7 @@ def checkout(ctx, group, ue: int):
 
         ep2_ex_test list ungraded --group <group> --ue <ue>
     """
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
 
     git_path = ep2.config.get("Local", "GitHome")
@@ -353,7 +410,8 @@ def checkout(ctx, group, ue: int):
 
 @cli.command()
 @click.option("--repo-owner", required=True, prompt=True, help='the matriculation number of the person, in whose '
-                                                               'repository the exercise was written in')
+                                                               'repository the exercise was written in',
+              callback=validate_student_id)
 @click.option("--points", required=True, prompt=True, help='points graded for this exercise', type=click.IntRange(0, 4))
 @click.option("--group", required=True, prompt=True, help='name of the group')
 @click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero')
@@ -364,7 +422,7 @@ def grade(ctx, repo_owner, points, ue, group, remarks, feedback):
     """This command grades a single submission by updating the submissions CSV entry and creating an
     issue for the project in which the ad-hoc exercise was written in."""
     f_info = FileInformation('Grade')
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
 
     csv_file = group.exercise_test_csv(ue)
@@ -441,7 +499,7 @@ def submit(ctx, group, ue):
 
     To prevent accidental issue creation before validation, a challenge has to be entered, which is printed
     at the top of the output."""
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
 
     f_info = FileInformation('Submit')
@@ -571,7 +629,7 @@ def list_grp():
 @click.pass_context
 def list_ungraded(ctx, ue, group):
     """This command lists all ungraded submission for a specific group and exercise."""
-    ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
 
     csv_file = group.exercise_test_csv(ue)
